@@ -14,7 +14,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: PicDevice.h,v 1.39 2004/08/22 16:28:51 marka Exp $
+ * $Id: PicDevice.h,v 1.42 2004/09/26 15:49:00 marka Exp $
  */
 #ifndef __PicDevice_h
 #define __PicDevice_h
@@ -38,6 +38,7 @@ typedef pair<int, int> IntPair;			/**< A pair of integers */
 #define PIC_REQUIRE_EPROG  0x00000002	/**< PIC requires END_PROG command */
 #define PIC_FEATURE_BKBUG  0x00000004	/**< PIC has on-chip debugger */
 #define PIC_HAS_OSCAL      0x00000008	/**< PIC has an OSCAL value that must be saved and restored on a chip erase */
+#define PIC_HAS_DEVICEID   0x00000010	/**< PIC has a device ID that can be read by the programming software */
 
 
 /** Macro to assign a default value if the configuration variable isn't set */
@@ -104,6 +105,8 @@ public:
 	/** Destructor */
 	~PicDevice();
 
+	virtual bool check(void);
+
 protected:
 	/** Perform a single program cycle for program memory. The following steps
 	 * are performed:
@@ -153,6 +156,14 @@ protected:
 	 */
 	virtual uint32_t read_prog_data(void);
 
+	/** Read the device ID from the device. 
+	 * \pre The device has just been put into program mode.
+	 * \post The device is in an undetermined state. Programming may not
+	 *       be possible unless program mode is exited and re-entered.
+	 * \returns The value of the device ID word.
+	 */
+	virtual uint32_t read_deviceid(void);
+	
 /* Protected data: */
 	/** The calculated bitmask for clearing upper bits of a word. */
 	uint32_t wordmask;
@@ -186,6 +197,10 @@ protected:
 	/** The amount of time (in microseconds) it takes to erase an electrically
 	 * erasable device. */
 	unsigned int erase_time;
+
+	/** Expected device ID value and mask for which bits to care about.
+	 * These are only valid if the PIC_HAS_DEVICEID flag is set. */
+	uint32_t deviceid, deviceidmask;
 };
 
 
@@ -352,6 +367,8 @@ protected:
 	 */
 	uint32_t read_ee_data(void);
 
+	virtual uint32_t read_deviceid(void);
+
 /* Protected data: */
 	/** Bitmask for valid bits in the configuration word. */
 	unsigned int config_mask;
@@ -633,6 +650,8 @@ protected:
 	 */
 	virtual unsigned int write_command_read_data(unsigned int command);
 
+	virtual uint32_t read_deviceid(void);
+
 /* Protected Data: */
 	/** The bitmasks for each configuration word */
 	unsigned int config_masks[7];
